@@ -9,6 +9,10 @@ from client_sdk_python.utils.toolz import (
     merge,
 )
 
+from hexbytes import HexBytes
+
+from eth_keys.datatypes import PrivateKey
+
 VALID_TRANSACTION_PARAMS = [
     'from',
     'to',
@@ -29,6 +33,40 @@ TRANSACTION_DEFAULTS = {
 }
 
 
+def call_obj(obj, from_address, to_address, data):
+    to_address = obj.web3.toChecksumAddress(to_address)
+    if from_address is None:
+        return obj.web3.platon.call({"to": to_address, "data": data})
+    from_address = obj.web3.toChecksumAddress(from_address)
+    return obj.web3.platon.call({"from": from_address, "to": to_address, "data": data})
+
+
+def send_obj_transaction(obj, data, to_address, pri_key, transaction_cfg: dict):
+    if transaction_cfg is None:
+        transaction_cfg = {}
+    if transaction_cfg.get("gasPrice", None) is None:
+        transaction_cfg["gasPrice"] = obj.web3.platon.gasPrice
+    if transaction_cfg.get("nonce", None) is None:
+        raw_from_address = PrivateKey(bytes.fromhex(pri_key)).public_key.to_address()
+        from_address = obj.web3.toChecksumAddress(raw_from_address)
+        transaction_cfg["nonce"] = obj.web3.platon.getTransactionCount(from_address)
+    if transaction_cfg.get("gas", None) is None:
+        transaction_data = {"to": to_address, "data": data}
+        transaction_cfg["gas"] = obj.web3.platon.estimateGas(transaction_data)
+    transaction_cfg["chainId"] = obj.web3.chainId
+    transaction_cfg["to"] = to_address
+    transaction_cfg["data"] = data
+    if transaction_cfg.get("value", 0) > 0:
+        transaction_cfg["value"] = int(transaction_cfg.get("value", 0))
+    signed_transaction_dict = obj.web3.platon.account.signTransaction(
+        transaction_cfg, pri_key
+    )
+    signed_data = signed_transaction_dict.rawTransaction
+    tx_hash = HexBytes(obj.web3.platon.sendRawTransaction(signed_data)).hex()
+    if obj.need_analyze:
+        return obj.web3.platon.analyzeReceiptByHash(tx_hash)
+    return tx_hash
+
 @curry
 def fill_nonce(web3, transaction):
     if 'from' in transaction and 'nonce' not in transaction:
@@ -137,8 +175,15 @@ def prepare_replacement_transaction(web3, current_transaction, new_transaction):
     if current_transaction['blockHash'] is not None:
         raise ValueError('Supplied transaction with hash {} has already been mined'
                          .format(current_transaction['hash']))
-    if 'nonce' in new_transaction and new_transaction['nonce'] != current_transaction['nonce']:
-        raise ValueError('Supplied nonce in new_transaction must match the pending transaction')
+
+    if 'nonce' in new_transaction:
+        if isinstance(new_transaction['nonce'], int) :
+            if new_transaction['nonce'] != int(current_transaction['nonce'], 16):
+                raise ValueError('Supplied nonce in new_transaction must match the pending transaction')
+        else:
+            if '0x' in new_transaction['nonce']:
+                if int(new_transaction['nonce'], 16) != int(current_transaction['nonce'], 16):
+                    raise ValueError('Supplied nonce in new_transaction must match the pending transaction')
 
     if 'nonce' not in new_transaction:
         new_transaction = assoc(new_transaction, 'nonce', current_transaction['nonce'])
